using System;
using System.Collections.Generic;
using System.Linq.Expressions;
using System.Threading.Tasks;
using MongoDB.Driver;
using Posts.WebApi.Models.Entities;

namespace Posts.WebApi.DataAccess
{
    public class BaseMongoDbRepository<TEntity> : IRepository<TEntity>
        where TEntity : IDocumentEntity
    {
        protected const int DuplicateCode = 11000;
        protected readonly IMongoDatabase _mongoDatabase;

        public BaseMongoDbRepository(IMongoDatabase mongoDatabase)
        {
            this._mongoDatabase = mongoDatabase;
        }

        protected string CollectionName => typeof(TEntity).Name;

        public async Task InsertAsync(TEntity entity)
        {
            try
            {
                await this._mongoDatabase.GetCollection<TEntity>(CollectionName)
                                    .InsertOneAsync(entity);
            }
            catch (MongoWriteException ex)
            {
                if (ex.WriteError.Code == DuplicateCode)
                {
                    throw new Exception("Dublicate");
                }

                throw new Exception(ex.Message);
            }
            catch (MongoException ex)
            {
                throw new Exception(ex.Message);
            }
        }

        public async Task UpdateAsync(TEntity entity)
        {
            try
            {
                var result = await this._mongoDatabase.GetCollection<TEntity>(CollectionName)
                                                 .ReplaceOneAsync(x => x.Id == entity.Id, entity);

                if (result.MatchedCount != 1)
                {
                    throw new Exception(entity.Id);
                }
            }
            catch (MongoException ex)
            {
                throw new Exception(ex.Message);
            }
        }

        public async Task<TEntity> GetByIdAsync(string id) =>
            await this._mongoDatabase.GetCollection<TEntity>(CollectionName)
                                .Find(x => x.Id == id)
                                .FirstOrDefaultAsync();

        public async Task<IEnumerable<TEntity>> GetByAsync(Expression<Func<TEntity, bool>> predicate) =>
            await this._mongoDatabase.GetCollection<TEntity>(CollectionName)
                         .Find(predicate)
                         .ToListAsync();

        public async Task<IEnumerable<TEntity>> GetAllAsync() =>
            await this._mongoDatabase.GetCollection<TEntity>(CollectionName)
                                .Find(x => true)
                                .ToListAsync();
    }
}