using System;
using System.Collections.Generic;
using System.Security;
using MongoDB.Bson.Serialization.Attributes;
using MongoDB.Bson;
using Posts.WebApi.Models.Requests;
using System.Linq;

namespace Posts.WebApi.Models.Entities
{
    public class Post : IDocumentEntity
    {
        [BsonId]
        [BsonRepresentation(BsonType.ObjectId)]
        public string Id { get; set; }
        public DateTime CreatedAtUtc { get; set; }

        public Post(string title = null, string description = null, int course = default, int semester = default,
            DateTime? givenAt = default, DateTime lastVisit = default,
            DateTime modifiedAtUtc = default, DateTime? deletedAtUtc = default, int? postValueTypeId = default,
            int groupId = default, int userId = default,
            int subjectId = default, DateTime createdAtUtc = default, IEnumerable<FileInfoRequest> files = default)
        {
            Title = title;
            Description = description;
            Course = course;
            Semester = semester;
            GivenAt = givenAt;
            LastVisit = lastVisit;
            ModifiedAtUtc = modifiedAtUtc;
            DeletedAtUtc = deletedAtUtc;
            PostValueTypeId = postValueTypeId;
            GroupId = groupId;
            UserId = userId;
            SubjectId = subjectId;
            CreatedAtUtc = createdAtUtc;
            Files = files.Select(f => new FileEntity(f)).ToList();
        }

        public Post()
        {
        }

        public string Title { get; set; }
        public string Description { get; set; }
        public int Course { get; set; }
        public int Semester { get; set; }
        public DateTime? GivenAt { get; set; }
        public DateTime LastVisit { get; set; }
        public DateTime ModifiedAtUtc { get; set; }
        public DateTime? DeletedAtUtc { get; set; }
        public DateTime? ValidatedAtUtc { get; set; }

        public int? PostValueTypeId { get; set; }

        public int GroupId { get; set; }

        public int UserId { get; set; }

        public int SubjectId { get; set; }

        public List<FileEntity> Files { get; set; }

        public Post UpdatePost(AddPostRequest request, IEnumerable<File> files, DateTime utcNow)
        {
            if (request == null)
            {
                throw new ArgumentNullException(nameof(request));
            }

            Title = request.Title;
            GroupId = request.GroupId;
            Description = request.Description;
            Semester = request.Semester;
            Course = request.Course;

            if (request.PostValueType != null)
            {
                PostValueTypeId = (int)request.PostValueType;
            }

            SubjectId = request.SubjectId;
            GivenAt = request.GivenAt;

            foreach (var file in files)
            {
                file.DeletedAtUtc = null;
            }

            return this;
        }

        public Post ValidatePost(DateTime utcNow)
        {
            if (this.ValidatedAtUtc == null)
            {
                ValidatedAtUtc = utcNow;
            }
            else
            {
                throw new VerificationException(); 
            }

            return this;
        }
    }
}