import { Injectable } from '@angular/core';
import { HttpRequest, HttpHandler, HttpEvent, HttpInterceptor } from '@angular/common/http';
import { Observable } from 'rxjs';
import { Store } from '@ngrx/store';

import * as fromStore from '../modules/authentication/model/store/index';

@Injectable()
export class JwtInterceptor implements HttpInterceptor {

    private token: string | undefined;

    constructor(private store: Store<fromStore.AuthenticationState>) {
        this.store.select(fromStore.getAccessToken).subscribe(t => {
            this.token = t;
        });
    }

    public intercept(request: HttpRequest<any>, next: HttpHandler): Observable<HttpEvent<any>> {
        if (this.token) {
            request = request.clone({
                setHeaders: {
                    Authorization: `Bearer ${this.token}`
                }
            });
        }
        return next.handle(request);
    }
}