import { Action } from '@ngrx/store';
import { UserLoginResponse } from '../../responses/user-login-response.model';
import { RefreshTokenResponse } from '../../responses/refresh-token-response.model';
import { UserProfile } from '@app/shared/models';

export enum ProfileActionTypes {
  RegistrationStart = '[Profile] Registration Start',
  RegistrationSuccess = '[Profile] Registration Success',
  RegistrationFailure = '[Profile] Registration Failure',
  LoginStart = '[Profile] Login Start',
  LoginSuccess = '[Profile] Login Success',
  LoginFailure = '[Profile] Login Failure',
  RefreshTokenStart = '[Profile] Refresh Token Start',
  RefreshTokenSuccess = '[Profile] Refresh Token Success',
  RefreshTokenFailure = '[Profile] Refresh Token Failure',
  SetInitialState = '[Profile] Set Initial State',
  ClearErrorMessage = '[Profile] Clear Error Message',
  GetMyProfileStart = '[Profile] Get My Profile Start',
  GetMyProfileSuccess = '[Profile] Get My Profile Success',
  GetMyProfileFailure = '[Profile] Get My Profile Failure',
  ClearRegistrationStatus = '[Profile]] Clear Registration Status',
  AutoLoginSuccess = "AutoLoginSuccess"
}

export class RegistrationStart implements Action {
  readonly type = ProfileActionTypes.RegistrationStart;
  constructor(public username: string, public email: string, public password: string) { }
}

export class RegistrationSuccess implements Action {
  readonly type = ProfileActionTypes.RegistrationSuccess;
  constructor() { }
}

export class RegistrationFailure implements Action {
  readonly type = ProfileActionTypes.RegistrationFailure;
  constructor(public errorMessage: string) { }
}

export class LoginStart implements Action {
  readonly type = ProfileActionTypes.LoginStart;
  constructor(public username: string, public password: string) { }
}

export class LoginSuccess implements Action {
  readonly type = ProfileActionTypes.LoginSuccess;
  constructor(public userLoginResponse: UserLoginResponse) { }
}

export class AutoLoginSuccess implements Action {
  readonly type = ProfileActionTypes.AutoLoginSuccess;
  constructor(public userLoginResponse: UserLoginResponse) { }
}

export class LoginFailure implements Action {
  readonly type = ProfileActionTypes.LoginFailure;
  constructor(public errorMessage: string) { }
}

export class RefreshTokenStart implements Action {
  readonly type = ProfileActionTypes.RefreshTokenStart;
}

export class RefreshTokenSuccess implements Action {
  readonly type = ProfileActionTypes.RefreshTokenSuccess;
  constructor(public refreshTokenResponse: RefreshTokenResponse) { }
}

export class RefreshTokenFailure implements Action {
  readonly type = ProfileActionTypes.RefreshTokenFailure;
  constructor(public errorMessage: string) { }
}

export class SetInitialState implements Action {
  readonly type = ProfileActionTypes.SetInitialState;
}

export class ClearErrorMessage implements Action {
  readonly type = ProfileActionTypes.ClearErrorMessage;
}

export class ClearRegistrationStatus implements Action {
  readonly type = ProfileActionTypes.ClearRegistrationStatus;
}

export class GetMyProfileStart implements Action {
  readonly type = ProfileActionTypes.GetMyProfileStart;
  constructor(public userId: number) { }
}

export class GetMyProfileSuccess implements Action {
  readonly type = ProfileActionTypes.GetMyProfileSuccess;
  constructor(public userProfile: UserProfile) { }
}

export class GetMyProfileFailure implements Action {
  readonly type = ProfileActionTypes.GetMyProfileFailure;
  constructor(public errorMessage: string) { }
}

export type ProfileActions
  = RegistrationStart
  | RegistrationSuccess
  | RegistrationFailure
  | LoginStart
  | LoginSuccess
  | AutoLoginSuccess
  | LoginFailure
  | RefreshTokenStart
  | RefreshTokenSuccess
  | RefreshTokenFailure
  | SetInitialState
  | ClearErrorMessage
  | GetMyProfileStart
  | GetMyProfileSuccess
  | GetMyProfileFailure
  | ClearRegistrationStatus;