import { Injectable } from '@angular/core';
import { Actions, Effect, ofType } from '@ngrx/effects';
import { AuthenticationService } from '../../../services/web/authentication/authentication.service';

import * as fromLogin from '../actions/profile.actions';
import { switchMap, map, catchError } from 'rxjs/operators';
import { UserLoginResponse } from '../../responses/user-login-response.model';
import { of } from 'rxjs';
import { UserProfileRestService } from '../../../services/web';



@Injectable()
export class ProfileEffects {
  constructor(
    private actions$: Actions,
    private authenticationService: AuthenticationService,
    private userProfileRestService: UserProfileRestService,
  ) { }

  @Effect()
  Login$ = this.actions$.pipe(
    ofType(fromLogin.ProfileActionTypes.LoginStart),
    switchMap(async (action: fromLogin.LoginStart) => {
      try {
        const userLoginResponse: UserLoginResponse = await this.authenticationService.login(action.username, action.password);
        return new fromLogin.LoginSuccess(userLoginResponse);
      } catch (error) {
        return new fromLogin.LoginFailure(error);
      }
    })
  );

  @Effect()
  Register$ = this.actions$.pipe(
    ofType(fromLogin.ProfileActionTypes.RegistrationStart),
    switchMap(async (action: fromLogin.RegistrationStart) => {
      try {
        await this.authenticationService.register(action.username, action.email, action.password);
        return new fromLogin.RegistrationSuccess();
      } catch (error) {
        return new fromLogin.RegistrationFailure(error);
      }
    })
  );

  @Effect()
  UserProfile$ = this.actions$.pipe(
    ofType(fromLogin.ProfileActionTypes.GetMyProfileStart),
    switchMap((action: fromLogin.GetMyProfileStart) => {
        return this.userProfileRestService.getUserByProfileId(action.userId)
          .pipe(
            map(userProfile => new fromLogin.GetMyProfileSuccess(userProfile)),
            catchError(response => of(new fromLogin.GetMyProfileFailure(response.message)))
          );
    })
  );
}
