import * as fromProfile from '../actions/profile.actions';
import { UserProfile } from '@app/shared/models';


export interface ProfileState {
  accessToken: string | undefined;
  refreshToken: string | undefined;
  expireAt: Date | undefined;
  userProfile: UserProfile | undefined;
  loaded: boolean;
  loading: boolean;
  errorMessage: string | undefined;
  isRegistrationSuccessful: boolean | undefined;
}

export const initialState: ProfileState = {
  accessToken: undefined,
  refreshToken: undefined,
  expireAt: undefined,
  userProfile: undefined,
  loaded: false,
  loading: false,
  errorMessage: undefined,
  isRegistrationSuccessful: false
};

export function reducer(state = initialState, action: fromProfile.ProfileActions): ProfileState {
  switch (action.type) {
    case fromProfile.ProfileActionTypes.RegistrationStart: {
      return {
        ...state,
        loading: true,
        loaded: false,
        errorMessage: undefined,
        isRegistrationSuccessful: false
      };
    }
    case fromProfile.ProfileActionTypes.RegistrationSuccess: {
      return {
        ...state,
        loading: false,
        loaded: true,
        errorMessage: undefined,
        isRegistrationSuccessful: true
      };
    }
    case fromProfile.ProfileActionTypes.RegistrationFailure: {
      return {
        ...state,
        loading: false,
        loaded: false,
        errorMessage: action.errorMessage,
        isRegistrationSuccessful: false
      };
    }
    case fromProfile.ProfileActionTypes.LoginStart: {
      return {
        ...state,
        loading: true,
        loaded: false,
        errorMessage: undefined
      };
    }
    case fromProfile.ProfileActionTypes.LoginSuccess: {
      return {
        ...state,
        loading: false,
        loaded: true,
        errorMessage: undefined,
        accessToken: action.userLoginResponse.accessToken,
        expireAt: action.userLoginResponse.expireAt,
        refreshToken: action.userLoginResponse.refreshToken,
        userProfile: action.userLoginResponse.userProfile
      };
    }
    case fromProfile.ProfileActionTypes.AutoLoginSuccess: {
      return {
        ...state,
        loading: false,
        loaded: true,
        errorMessage: undefined,
        accessToken: action.userLoginResponse.accessToken,
        expireAt: action.userLoginResponse.expireAt,
        refreshToken: action.userLoginResponse.refreshToken,
        userProfile: action.userLoginResponse.userProfile
      };
    }
    case fromProfile.ProfileActionTypes.LoginFailure: {
      return {
        ...state,
        loading: false,
        loaded: false,
        errorMessage: action.errorMessage
      };
    }
    case fromProfile.ProfileActionTypes.RefreshTokenStart: {
      return {
        ...state,
        loading: true,
        loaded: false,
        errorMessage: undefined
      };
    }
    case fromProfile.ProfileActionTypes.RefreshTokenSuccess: {
      return {
        ...state,
        loading: false,
        loaded: true,
        errorMessage: undefined,
        accessToken: action.refreshTokenResponse.accessToken,
        expireAt: action.refreshTokenResponse.expireAt,
        refreshToken: action.refreshTokenResponse.refreshToken
      };
    }
    case fromProfile.ProfileActionTypes.RefreshTokenFailure: {
      return {
        ...state,
        loading: false,
        loaded: false,
        errorMessage: action.errorMessage
      };
    }
    case fromProfile.ProfileActionTypes.ClearErrorMessage: {
      return {
        ...state,
        errorMessage: undefined
      }
    }
    case fromProfile.ProfileActionTypes.ClearRegistrationStatus: {
      return {
        ...state,
        isRegistrationSuccessful: undefined
      }
    }
    case fromProfile.ProfileActionTypes.SetInitialState: {
      return initialState;
    }
    case fromProfile.ProfileActionTypes.GetMyProfileStart: {
      return {
        ...state,
        loading: true,
        loaded: false,
        errorMessage: undefined,
      };
    }
    case fromProfile.ProfileActionTypes.GetMyProfileSuccess: {
      return {
        ...state,
        loading: false,
        loaded: true,
        errorMessage: undefined,
        userProfile: action.userProfile
      };
    }
    case fromProfile.ProfileActionTypes.GetMyProfileFailure: {
      return {
        ...state,
        loading: false,
        loaded: false,
        errorMessage: action.errorMessage
      };
    }
    default:
      return state;
  }
}
