import { AuthenticationWebServiceToken, AuthenticationWebService } from './authentication-web.service';
import { Inject, Injectable } from '@angular/core';
import { UserLoginResponse } from '@app/core/modules/authentication/model/responses/user-login-response.model';

@Injectable()
export class AuthenticationService {
    constructor (@Inject(AuthenticationWebServiceToken) private authenticationWebService: AuthenticationWebService) {}

    public async login(email: string, password: string): Promise<UserLoginResponse> {
        return new Promise(async (resolve, reject) => {
            resolve(UserLoginResponse.assemble(await this.authenticationWebService.login(email, password)
            .catch((error : any) => reject(this.handleErrorMessage(error)))
            ));
        });
    }

    public async register(username: string, email: string, password: string, description?: string): Promise<any> {
        return new Promise(async (resolve, reject) => {
            resolve(UserLoginResponse.assemble(await this.authenticationWebService.register(email, password, username, description)
            .catch((error : any) => reject(this.handleErrorMessage(error)))));
        });
    }

    private handleErrorMessage(error: any): string {
        let errorMessage: string;
        if(error.error && typeof(error.error) === 'string') {
            errorMessage = error.error;
        } else {
            errorMessage = 'Something went wrong. Try again later'
        }
        return errorMessage;
    }
}