import { Injectable } from '@angular/core';
import { HttpHeaders, HttpClient } from '@angular/common/http';

import { environment } from '@environments/environment';

import { PostFull } from '@app/shared/models/post-full';
import { Observable } from 'rxjs';
import { NewPost } from '@app/shared/models/new-post';
import { Post } from '@app/shared/models';
import { Complaint } from '@app/shared/models/complaint';
import { PostProfile } from '@app/shared/models/post-profile';
import { map } from 'rxjs/operators';

const httpOptions = {
  headers: new HttpHeaders({
    'Content-Type': 'application/json',
  }),
};

@Injectable()
export class PostService {
  constructor(private http: HttpClient) {}

  public getPostFullInfo(postId: string): Observable<PostFull> {
    return this.http.get<PostFull>(environment.urlPostsServerAddress + 'posts/' + postId);
  }

  public addPost(post: NewPost): Observable<Post> {
    return this.http.post<Post>(environment.urlPostsServerAddress + 'posts', post, httpOptions);
  }

  public editPost(post: NewPost, postId: string): Observable<Post> {
    return this.http.put<Post>(environment.urlPostsServerAddress + 'posts/' + postId, post, httpOptions);
  }

  public deletePost(postId: string): Observable<object> {
    return this.http.delete(environment.urlPostsServerAddress + 'posts/' + postId, httpOptions);
  }

  public verifyPost(postId: string): Observable<object> {
    return this.http.patch(environment.urlPostsServerAddress + 'posts/' + postId + '/validate', httpOptions);
  }

  public addComplaintForPost(postId: string, complaintText: string): Observable<Complaint> {
    var addComplaintReuqest: any = {
      postId: postId,
      complaintText: complaintText,
    };

    return this.http.post<Complaint>(environment.urlPostsServerAddress + 'complaints', addComplaintReuqest, httpOptions);
  }

  public getUsersPosts(skipNumber: number, takeNumber: number): Observable<PostProfile[]> {
    return this.http
      .get<any[]>(environment.urlPostsServerAddress + 'posts/user' + '?skip=' + skipNumber + '&take=' + takeNumber, httpOptions)
      .pipe(
        map(postDto => {
          const result: PostProfile[] = Array();
          if (postDto) {
            postDto.forEach(value => {
              const post: PostProfile = {
                id: value.id,
                description: value.description,
                groupId: value.groupId,
                groupTitle: value.groupTitle,
                lastVisit: value.lastVisit,
                postLocationType: value.postLocationType,
                postValueType: value.postValueType,
                semester: value.semester,
                subjectId: value.subjectId,
                subjectTitle: value.subjectTitle,
                teacherName: value.teacherName,
                title: value.title,
                userProfileId: value.userProfileId,
                validatedAtUtc: value.validatedAtUtc,
              };
              result.push(post);
            });
          }
          return result;
        }),
      );
  }
}
