import { Action } from '@ngrx/store';
import { PostFull } from '@app/shared/models/post-full';

export enum PostActionTypes {
  LoadPostStart = '[Post] LoadPostStart',
  LoadPostSuccess = '[Post] LoadPostSuccess',
  LoadPostFailure = '[Post] LoadPostFailure',
  VotePostStart = '[Post] VotePostStart',
  VotePostSuccess = '[Post] VotePostSuccess',
  VotePostFailure = '[Post] VotePostFailure',
  ReportPostStart = '[Post] ReportPostStart',
  ReportPostSuccess = '[Post] ReportPostSuccess',
  ReportPostFailure = '[Post] ReportPostFailure',
  DeletePostStart = '[Post] DeletePostStart',
  VerifyPostStart = '[Post] VerifyPostStart',
  DeletePostSuccess = '[Post] DeletePostSuccess',
  DeletePostFailure = '[Post] DeletePostFailure',
  VerifyPostSuccess = '[Post] VerifyPostSuccess',
  VerifyPostFailure = '[Post] VerifyPostFailure',
  ClearPost = '[Post] Clear post',
}

export class LoadPostStart implements Action {
  readonly type = PostActionTypes.LoadPostStart;
  constructor(public postId: string) {}
}

export class LoadPostSuccess implements Action {
  readonly type = PostActionTypes.LoadPostSuccess;
  constructor(public postFull: PostFull) {}
}

export class LoadPostFailure implements Action {
  readonly type = PostActionTypes.LoadPostFailure;
  constructor(public message: string) {}
}

export class ClearPost implements Action {
  readonly type = PostActionTypes.ClearPost;
}

export class ReportPostStart implements Action {
  readonly type = PostActionTypes.ReportPostStart;
  constructor(public postId: string, public complain: string) {}
}

export class ReportPostSuccess implements Action {
  readonly type = PostActionTypes.ReportPostSuccess;
  constructor() {}
}

export class ReportPostFailure implements Action {
  readonly type = PostActionTypes.ReportPostFailure;
  constructor(public message: string) {}
}

export class DeletePostStart implements Action {
  readonly type = PostActionTypes.DeletePostStart;
  constructor(public postId: string) {}
}

export class VerifyPostStart implements Action {
  readonly type = PostActionTypes.VerifyPostStart;
  constructor(public postId: string) {}
}

export class DeletePostSuccess implements Action {
  readonly type = PostActionTypes.DeletePostSuccess;
  constructor() {}
}

export class DeletePostFailure implements Action {
  readonly type = PostActionTypes.DeletePostFailure;
  constructor(public message: string) {}
}

export class VerifyPostSuccess implements Action {
  readonly type = PostActionTypes.VerifyPostSuccess;
  constructor() {}
}

export class VerifyPostFailure implements Action {
  readonly type = PostActionTypes.VerifyPostFailure;
  constructor(public message: string) {}
}

export type PostActions =
  | LoadPostStart
  | LoadPostSuccess
  | LoadPostFailure
  | ClearPost
  | ReportPostStart
  | ReportPostSuccess
  | ReportPostFailure
  | DeletePostStart
  | DeletePostSuccess
  | DeletePostFailure
  | VerifyPostStart
  | VerifyPostSuccess
  | VerifyPostFailure;
