import { Component } from '@angular/core';
import { Router } from '@angular/router';
import { Faculty } from '@app/shared/models/index';
import * as fromStore from '../../../model/store';
import { Store } from '@ngrx/store';
import { Observable } from 'rxjs';
import { University } from '@app/shared/models/university';
import { take } from 'rxjs/operators';

const DEFAULT_LOAD_SIZE = 20;

@Component({
  selector: 'app-faculty-select',
  templateUrl: './faculty-select-root.component.html',
  styleUrls: ['./faculty-select-root.component.scss'],
})
export class FacultySelectRootComponent {
  public faculties$: Observable<Faculty[]>;
  public loading$: Observable<boolean>;

  constructor(private store: Store<fromStore.SearchState>, private router: Router) {
    this.faculties$ = store.select(fromStore.getAllFaculties);
    this.loading$ = store.select(fromStore.getFacultyLoading);
  }

  public onScroll() {
    this.loadNext();
  }

  public onSelectFaculty(selectedFaculty: Faculty) {
    this.store.dispatch(new fromStore.SelectFaculty(selectedFaculty));
    this.router.navigate(['select/subject']);
  }

  private loadNext() {
    let selectedUniversity: University | undefined;
    let facultyCount: number | null = null;
    this.store
      .select(fromStore.getSelectedUniversity)
      .pipe(take(1))
      .subscribe(val => (selectedUniversity = val));
    this.store
      .select(fromStore.getFacultiesCount)
      .pipe(take(1))
      .subscribe(val => (facultyCount = val));
    if (selectedUniversity && facultyCount != null)
      this.store.dispatch(new fromStore.LoadFacultiesStart(selectedUniversity.id, DEFAULT_LOAD_SIZE, facultyCount));
  }
}
