import { Component } from '@angular/core';
import { Subject } from '@app/shared/models/subject';
import { Observable } from 'rxjs';
import { SearchState } from '@app/modules/search/model/store';
import { Store } from '@ngrx/store';
import * as fromStore from '../../../model/store';
import { Router } from '@angular/router';
import { Faculty } from '@app/shared/models';
import { take } from 'rxjs/operators';

const DEFAULT_LOAD_SIZE = 25;

@Component({
  selector: 'app-subject-select-root',
  templateUrl: './subject-select-root.component.html',
  styleUrls: ['./subject-select-root.component.scss'],
})
export class SubjectSelectRootComponent {
  public subjects$: Observable<Subject[]>;
  public loading$: Observable<boolean>;

  constructor(private store: Store<SearchState>, private router: Router) {
    this.subjects$ = store.select(fromStore.getAllSubjects);
    this.loading$ = store.select(fromStore.getSubjectLoading);
  }

  public onScroll() {
    this.loadNext();
  }

  private loadNext() {
    let selectedFaculty: Faculty | undefined;
    let subjectsCount: number | null = null;
    this.store
      .select(fromStore.getSelectedFaculty)
      .pipe(take(1))
      .subscribe(val => (selectedFaculty = val));
    this.store
      .select(fromStore.getSubjectCount)
      .pipe(take(1))
      .subscribe(val => (subjectsCount = val));
    if (selectedFaculty && subjectsCount != null)
      this.store.dispatch(new fromStore.LoadSubjectsStart(selectedFaculty.id, subjectsCount, DEFAULT_LOAD_SIZE));
  }

  public onSubjectSelect(subject: Subject): void {
    this.store.dispatch(new fromStore.SelectSubject(subject));
    this.router.navigate(['select/post']);
  }
}
