import { Component, Input, EventEmitter, Output } from '@angular/core';
import { Country, City } from '@app/shared/models';
import { Observable } from 'rxjs';

@Component({
  selector: 'app-uni-search-filter',
  templateUrl: './uni-search-filter.component.html',
  styleUrls: ['./uni-search-filter.component.scss'],
})
export class UniSearchFilterComponent {
  @Input() countries$: Observable<Country[]>;
  @Input() cities$: Observable<City[]>;
  @Input() selectedCountry$: Observable<Country>;
  @Input() selectedCity$: Observable<City>;

  @Output() selectCountry = new EventEmitter<Country | undefined>();
  @Output() selectCity = new EventEmitter<City | undefined>();

  public countryDisplayFunction = (country: Country | undefined): string => (country ? country.title : '');
  public cityDisplayFunction = (city: City | undefined): string => (city ? city.title : '');
  public compareCitiesFunction = (city1: City, city2: City): boolean => (city1 && city2 ? city1.id === city2.id : false);
  public compareCountriesFunction = (country1: Country, country2: Country): boolean =>
    country1 && country2 ? country1.id === country2.id : false;

  public onCountryChange(country: Country): void {
    this.selectCountry.emit(country);
  }

  public onCityChange(city: City): void {
    this.selectCity.emit(city);
  }

  public clearAllFilters(): void {
    this.selectCity.emit(undefined);
    this.selectCountry.emit(undefined);
  }
}
