import { Component } from '@angular/core';
import { City } from '@app/shared/models/city';
import { University } from '@app/shared/models/university';
import { Store } from '@ngrx/store';
import { merge, Observable } from 'rxjs';
import { take } from 'rxjs/operators';
import * as fromStore from '../../../model/store/index';
import { PopupService } from '@app/core/services';
import { Country } from '@app/shared/models';
import { Router } from '@angular/router';

@Component({
  selector: 'app-uni-search-root',
  templateUrl: './uni-search-root.component.html',
  styleUrls: ['./uni-search-root.component.scss'],
})
export class UniSearchRootComponent {
  public countries$: Observable<Country[]>;
  public cities$: Observable<City[]>;
  public universities$: Observable<University[]>;
  public selectedCountry$: Observable<Country | undefined>;
  public selectedCity$: Observable<City | undefined>;
  public loading$: Observable<boolean>;

  constructor(private store: Store<fromStore.SearchState>, private popupService: PopupService, private router: Router) {
    this.universities$ = this.store.select(fromStore.getAllUniversities);
    this.selectedCountry$ = this.store.select(fromStore.getSelectedCountry);
    this.selectedCity$ = this.store.select(fromStore.getSelectedCity);
    this.countries$ = store.select(fromStore.getAllCountries);
    this.cities$ = store.select(fromStore.getAllCities);

    this.loading$ = merge(this.store.select(fromStore.getCityLoading), this.store.select(fromStore.getCountryLoading));

    merge(this.store.select(fromStore.getCountryErrorMessage), this.store.select(fromStore.getCityErrorMessage)).subscribe(value => {
      if (value) this.popupService.showError(value);
    });

    this.store.dispatch(new fromStore.LoadCountryStart());
  }

  public onCountryChange(country: Country | undefined): void {
    this.store.dispatch(new fromStore.SelectCountry(country));
    if (country) {
      this.store.dispatch(new fromStore.LoadCityStart(country.id));
    }
  }

  public onCityChange(city: City | undefined): void {
    this.store.dispatch(new fromStore.SelectCity(city));
    if (city) {
      this.loadNext();
    }
  }

  public onUniversitySelect(university: University): void {
    this.store.dispatch(new fromStore.SelectUniversity(university));
    this.router.navigate(['select/faculty/']);
  }

  public loadNext(): void {
    let selectedCity: City | undefined;
    let cityCount: number | null = null;
    this.store
      .select(fromStore.getSelectedCity)
      .pipe(take(1))
      .subscribe(val => (selectedCity = val));
    this.store
      .select(fromStore.getUniversitiesCount)
      .pipe(take(1))
      .subscribe(val => (cityCount = val));
    if (selectedCity && cityCount != null) this.store.dispatch(new fromStore.LoadUniversitiesStart(selectedCity.id, cityCount));
  }
}
