import { Action } from '@ngrx/store';
import { Country } from '@app/shared/models';

export enum SearchCountryActionTypes {
  LoadCountryStart = '[SearchCountry] Load Country Start',
  LoadCountrySuccess = '[SearchCountry] Load Country Success',
  LoadCountryFailure = '[SearchCountry] Load Country Failure',
  SelectCountry = '[SearchCountry] Select Country',
  SetSearchCountryInitialState = '[SearchCountry] Set Initial State',
}

export class LoadCountryStart implements Action {
  readonly type = SearchCountryActionTypes.LoadCountryStart;
  constructor() {}
}

export class LoadCountrySuccess implements Action {
  readonly type = SearchCountryActionTypes.LoadCountrySuccess;
  constructor(public countries: Country[]) {}
}

export class LoadCountryFailure implements Action {
  readonly type = SearchCountryActionTypes.LoadCountryFailure;
  constructor(public errorMessage: string) {}
}

export class SelectCountry implements Action {
  readonly type = SearchCountryActionTypes.SelectCountry;
  constructor(public country: Country | undefined) {}
}

export class SetSearchCountryInitialState implements Action {
  readonly type = SearchCountryActionTypes.SetSearchCountryInitialState;
}

export type SearchCountryAction = Action &
  (LoadCountrySuccess | LoadCountryStart | LoadCountryFailure | SelectCountry | SetSearchCountryInitialState);
