import { Action } from '@ngrx/store';
import { Faculty } from '@app/shared/models';

export enum FacultyActionTypes {
  LoadFacultiesStart = '[SearchFaculty] Load Faculties Start',
  LoadFacultiesSuccess = '[SearchFaculty] Load Faculties Success',
  LoadFacultiesFailure = '[SearchFaculty] Load Faculties Failure',
  ClearFaculties = '[SearchFaculty] Clear Faculties',
  SelectFaculty = '[SearchFaculty] Select Faculty',
  SetFacultyInitialState = '[SearchFaculty] Set Initial State',
}

export class LoadFacultiesStart implements Action {
  readonly type = FacultyActionTypes.LoadFacultiesStart;
  constructor(public universityId: number, public take: number, public skip: number) {}
}

export class LoadFacultiesSuccess implements Action {
  readonly type = FacultyActionTypes.LoadFacultiesSuccess;
  constructor(public faculties: Faculty[]) {}
}

export class LoadFacultiesFailure implements Action {
  readonly type = FacultyActionTypes.LoadFacultiesFailure;
  constructor(public errorMessage: string) {}
}

export class ClearFaculties implements Action {
  readonly type = FacultyActionTypes.ClearFaculties;
}

export class SelectFaculty implements Action {
  readonly type = FacultyActionTypes.SelectFaculty;
  constructor(public faculty: Faculty | undefined) {}
}

export class SetFacultyInitialState implements Action {
  readonly type = FacultyActionTypes.SetFacultyInitialState;
}

export type FacultyActions =
  | LoadFacultiesStart
  | LoadFacultiesSuccess
  | LoadFacultiesFailure
  | ClearFaculties
  | SelectFaculty
  | SetFacultyInitialState;
