import { Injectable } from '@angular/core';
import { Actions, Effect, ofType } from '@ngrx/effects';
import { Store } from '@ngrx/store';
import { filter, switchMap, tap, map, catchError } from 'rxjs/operators';
import { CountryService } from '../../../services/web/country.service';
import * as fromCountry from '../actions/search-country.actions';
import * as fromCity from '../actions/search-city.actions';
import * as fromUniversity from '../actions/search-university.actions';
import { SearchUniversityState } from '../reducers/search-university.reducer';
import { of } from 'rxjs';

@Injectable()
export class CountryEffects {
  constructor(private actions$: Actions, private countryService: CountryService, private store: Store<SearchUniversityState>) {}

  @Effect()
  loadCountry = this.actions$.pipe(
    ofType(fromCountry.SearchCountryActionTypes.LoadCountryStart),
    switchMap(() => {
      return this.countryService.getCountries().pipe(
        map(result => new fromCountry.LoadCountrySuccess(result)),
        catchError(message => of(new fromCountry.LoadCountryFailure(message))),
      );
    }),
  );

  @Effect()
  clearUniversitiesAndSelectedCity = this.actions$.pipe(
    ofType(fromCountry.SearchCountryActionTypes.SelectCountry, fromCountry.SearchCountryActionTypes.SetSearchCountryInitialState),
    tap((action: fromCountry.SelectCountry | fromCountry.SetSearchCountryInitialState) => {
      if (
        action instanceof fromCountry.SetSearchCountryInitialState ||
        (action instanceof fromCountry.SelectCountry && action.country === undefined)
      ) {
        this.store.dispatch(new fromUniversity.ClearUniversities());
        this.store.dispatch(new fromCity.SetSearchCityInitialState());
      }
    }),
    filter(() => false),
  );
}
