import { Injectable } from '@angular/core';
import { Actions, Effect, ofType } from '@ngrx/effects';
import { FacultyService } from '../../../services/web/faculty.service';
import * as fromStore from '../../store/actions/';
import { switchMap, map, catchError, tap, filter } from 'rxjs/operators';
import { of } from 'rxjs';
import { SearchState } from '../reducers';
import { Store } from '@ngrx/store';

const SubjectFirstLoadDefaultSize = 50;

@Injectable()
export class SearchFacultyEffects {
  constructor(private actions$: Actions, private store: Store<SearchState>, private facultyService: FacultyService) {}

  @Effect()
  loadFaculty = this.actions$.pipe(
    ofType(fromStore.FacultyActionTypes.LoadFacultiesStart),
    switchMap((action: fromStore.LoadFacultiesStart) => {
      return this.facultyService.getFaculties(action.universityId, action.skip, action.take).pipe(
        map(faculties => new fromStore.LoadFacultiesSuccess(faculties)),
        catchError(response => of(new fromStore.LoadFacultiesFailure(response.message))),
      );
    }),
  );

  @Effect()
  selectFaculty = this.actions$.pipe(
    ofType(fromStore.FacultyActionTypes.SelectFaculty),
    tap((action: fromStore.SelectFaculty) => {
      if (action.faculty) {
        this.store.dispatch(new fromStore.LoadSubjectsStart(action.faculty.id, 0, SubjectFirstLoadDefaultSize));
      }
    }),
    filter(() => false),
  );
}
