import { Injectable } from '@angular/core';
import * as fromPostActions from '../actions/search-post.actions';
import { Actions, Effect, ofType } from '@ngrx/effects';
import { switchMap, map, catchError } from 'rxjs/operators';
import { of } from 'rxjs';
import { PostsService } from '@app/modules/search/services/web/posts.service';
import { PostByGroupSemester } from '@app/shared/models';

@Injectable()
export class SearchPostEffects {
  constructor(private actions$: Actions, private postService: PostsService) {}

  @Effect()
  loadInitialPosts = this.actions$.pipe(
    ofType(fromPostActions.SearchPostActionTypes.LoadInitialPostsStart),
    switchMap((action: fromPostActions.LoadInitialPostsStart) => {
      return this.postService.getInitialPosts(action.subjectId).pipe(
        map((result: PostByGroupSemester[]) => new fromPostActions.LoadInitialPostsSuccess(result)),
        catchError((result: string) => of(new fromPostActions.LoadInitialPostsFailure(result))),
      );
    }),
  );
}
