import { Injectable } from '@angular/core';
import { Actions, Effect, ofType } from '@ngrx/effects';
import * as fromStore from '../../store/actions/';
import { switchMap, map, catchError, tap, filter } from 'rxjs/operators';
import { of } from 'rxjs';
import { SearchState } from '../reducers';
import { Store } from '@ngrx/store';
import { SubjectWebService } from '@app/modules/search/services/web/subject.service';

@Injectable()
export class SearchSubjectEffects {
  constructor(private actions$: Actions, private store: Store<SearchState>, private subjectService: SubjectWebService) {}

  @Effect()
  loadSubjects = this.actions$.pipe(
    ofType(fromStore.SearchSubjectActionTypes.LoadSubjectsStart),
    switchMap((action: fromStore.LoadSubjectsStart) => {
      return this.subjectService.getSubjects(action.facultyId, action.skip, action.take).pipe(
        map(subjects => new fromStore.LoadSubjectsSuccess(subjects)),
        catchError(response => of(new fromStore.LoadSubjectsFailure(response.message))),
      );
    }),
  );

  @Effect()
  selectSubject = this.actions$.pipe(
    ofType(fromStore.SearchSubjectActionTypes.SelectSubject),
    tap((action: fromStore.SelectSubject) => {
      //TODO: delete double loading of posts
      if (action.subject) {
        this.store.dispatch(new fromStore.LoadInitialPostsStart(action.subject.id));
      }
    }),
    filter(() => false),
  );
}
