import { EntityState, EntityAdapter, createEntityAdapter } from '@ngrx/entity';
import { Faculty } from '@app/shared/models';
import { FacultyActions, FacultyActionTypes } from '../actions/search-faculty.actions';

export interface SearchFacultyState extends EntityState<Faculty> {
  loading: boolean;
  errorMessage: string | undefined;
  selectedFaculty: Faculty | undefined;
}

export const adapter: EntityAdapter<Faculty> = createEntityAdapter<Faculty>();

export const initialState: SearchFacultyState = adapter.getInitialState({
  loading: false,
  errorMessage: undefined,
  selectedFaculty: undefined,
} as SearchFacultyState);

export function reducer(state = initialState, action: FacultyActions): SearchFacultyState {
  switch (action.type) {
    case FacultyActionTypes.LoadFacultiesStart: {
      return {
        ...state,
        errorMessage: undefined,
        loading: true,
      };
    }
    case FacultyActionTypes.LoadFacultiesSuccess: {
      return adapter.addMany(action.faculties, { ...state, loading: false });
    }
    case FacultyActionTypes.LoadFacultiesFailure: {
      return {
        ...state,
        errorMessage: action.errorMessage,
        loading: false,
      };
    }
    case FacultyActionTypes.ClearFaculties: {
      return adapter.removeAll(state);
    }
    case FacultyActionTypes.SelectFaculty: {
      return { ...state, selectedFaculty: action.faculty };
    }
    case FacultyActionTypes.SetFacultyInitialState:
      return initialState;
    default:
      return state;
  }
}

export const { selectIds, selectEntities, selectAll, selectTotal } = adapter.getSelectors();
