import { Subject } from '@app/shared/models/subject';
import { EntityState, EntityAdapter, createEntityAdapter } from '@ngrx/entity';
import * as fromAction from '../actions/search-subject.actions';

export interface SearchSubjectState extends EntityState<Subject> {
  loading: boolean;
  loaded: boolean;
  selectedSubject: Subject | undefined;
  errorMessage: string | undefined;
}

export const adapter: EntityAdapter<Subject> = createEntityAdapter<Subject>();

export const initialState: SearchSubjectState = adapter.getInitialState({
  loaded: false,
  loading: false,
  selectedSubject: undefined,
  errorMessage: undefined,
});

export function reducer(state = initialState, action: fromAction.SearchSubjectActions): SearchSubjectState {
  switch (action.type) {
    case fromAction.SearchSubjectActionTypes.LoadSubjectsStart: {
      return { ...state, loading: true, loaded: false, errorMessage: undefined };
    }
    case fromAction.SearchSubjectActionTypes.LoadSubjectsSuccess: {
      return adapter.addMany(action.subjects, { ...state, loaded: true, loading: false });
    }
    case fromAction.SearchSubjectActionTypes.LoadSubjectsFailure: {
      return { ...state, errorMessage: action.message };
    }
    case fromAction.SearchSubjectActionTypes.SelectSubject: {
      return { ...state, selectedSubject: action.subject };
    }
    case fromAction.SearchSubjectActionTypes.SetSubjectInitialState:
      return initialState;
    default:
      return state;
  }
}

export const { selectIds, selectEntities, selectAll, selectTotal } = adapter.getSelectors();
