import { Component, OnDestroy } from '@angular/core';
import { Router, NavigationError, NavigationEnd } from '@angular/router';

import { Store } from '@ngrx/store';
import { UserProfile } from '@app/shared/models';

import * as fromStore from '@core/modules/authentication/model/store/index';
import { TranslateService } from '@ngx-translate/core';
import * as fromAuthStore from '@core/modules/authentication/model/store/index';
import { UntilDestroy, untilDestroyed } from '@ngneat/until-destroy';

@UntilDestroy()
@Component({
  selector: 'app-header',
  templateUrl: './header.component.html',
  styleUrls: ['./header.component.scss'],
})
export class HeaderComponent implements OnDestroy {
  public userProfile: UserProfile | undefined;
  private universityShortTitle: string | undefined;
  private facultyShortTitle: string | undefined;
  // TODO use subject for post select
  private subjectFullTitle: string | undefined;

  constructor(
    private router: Router,
    private store: Store<fromStore.ProfileState>,
    private translateService: TranslateService,
    private storeProfile: Store<fromAuthStore.ProfileState>,
  ) {
    this.getUserProfile();
    this.subscribeRouterEvents();
  }

  public buttonProfileClick() {
    if (this.userProfile) {
      console.log('buttonProfileClick');
      this.router.navigate(['/profile/' + this.userProfile.id]);
    }
  }

  public buttonLogoutClick() {
    this.store.dispatch(new fromStore.SetInitialState());
    this.router.navigate(['/authorization']);
  }

  public buttonSettings() {
    this.router.navigate(['/profile/settings']);
  }

  public OnSelectClick() {
    if (this.subjectFullTitle) {
      this.router.navigate(['/select/post']);
    } else if (this.facultyShortTitle) {
      this.router.navigate(['/select/subject']);
    } else if (this.universityShortTitle) {
      this.router.navigate(['/select/faculty']);
    } else {
      this.router.navigate(['/select/university']);
    }
  }

  public useLanguage(language: string) {
    this.translateService.use(language);
  }

  public ngOnDestroy() {}

  private getUserProfile() {
    this.store
      .select(fromStore.getUserProfile)
      .pipe(untilDestroyed(this))
      .subscribe(value => (this.userProfile = value ? value : undefined));
  }

  private subscribeRouterEvents() {
    this.router.events.subscribe(event => {
      if (event instanceof NavigationEnd) {
        if (this.userProfile != undefined && this.userProfile.id != 0) {
          this.storeProfile.dispatch(new fromAuthStore.GetMyProfileStart(this.userProfile.id));
        }
      }

      if (event instanceof NavigationError) {
        console.log(event.error);
      }
    });
  }
}
