import { Params, RouterStateSnapshot } from '@angular/router';
import { routerReducer, RouterReducerState, RouterStateSerializer} from '@ngrx/router-store';
import { ActionReducerMap, createFeatureSelector, createSelector } from '@ngrx/store';
import { Injectable } from "@angular/core";

// This file creates global routerStore mechanism
export interface RouterState {
    routerState: RouterReducerState<RouterStateUrl>
}

export interface RouterStateUrl {
    url: string,
    params: Params,
    queryParams: Params
}

export const reducers: ActionReducerMap<RouterState> = {
    routerState: routerReducer
}

@Injectable()
export class CustomRouterSerializer implements RouterStateSerializer {
    serialize(routerState: RouterStateSnapshot): RouterStateUrl {
        return {
            params: routerState.root.params,
            queryParams: routerState.root.queryParams,
            url: routerState.url
        } as RouterStateUrl;
    }
}

export const selectRouter = createFeatureSelector<
RouterState,
RouterReducerState<RouterStateUrl>
>('routerState');

export const getRouterState = createSelector(
    selectRouter,
    (state) => state.state
);
