using System;
using System.IO;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.Options;
using Uni.WebApi.ApplicationCore.Helpers.Contract;
using Uni.WebApi.Common.Options;
using Uni.WebApi.Models.Enums;

namespace Uni.WebApi.ApplicationCore.Helpers
{
    public class FolderHelper : IFolderHelper
    {
        private readonly IWebHostEnvironment _hostingEnvironment;
        private readonly FilesOptions _filesOptions;

        public FolderHelper(IWebHostEnvironment hostingEnvironment, IOptions<FilesOptions> filesOptions)
        {
            this._hostingEnvironment = hostingEnvironment;
            this._filesOptions = filesOptions.Value;
        }

        public void CreateFilesFoldersIfNotExist()
        {
            var folderPath = Path.Combine(this._hostingEnvironment.ContentRootPath, $"{this._filesOptions.UploadFolder}");
            CheckOrCreateFolder(folderPath);
            CheckOrCreateFolder(Path.Combine(folderPath, $"{this._filesOptions.InnerFolders.ImagesFolder}"));
            CheckOrCreateFolder(Path.Combine(folderPath, $"{this._filesOptions.InnerFolders.FilesFolder}"));
            CheckOrCreateFolder(Path.Combine(folderPath, $"{this._filesOptions.InnerFolders.DefaultImagesFolder}"));

            foreach (EFileType @enum in Enum.GetValues(typeof(EFileType)))
            {
                var enumString = @enum.ToString();
                CheckOrCreateFolder(Path.Combine(folderPath, $"{this._filesOptions.InnerFolders.FilesFolder}/{enumString}"));
            }
        }

        private void CheckOrCreateFolder(string folderPath)
        {
            if (!Directory.Exists(folderPath))
            {
                Directory.CreateDirectory(folderPath);
            }
        }
    }
}