using System.Collections.Generic;
using System.Threading.Tasks;
using AutoMapper;
using Microsoft.Extensions.Options;
using Uni.WebApi.ApplicationCore.Constants;
using Uni.WebApi.ApplicationCore.Services.Contract;
using Uni.WebApi.Common.Options;
using Uni.WebApi.DataAccess.RepositoryService;
using Uni.WebApi.Kernel.Entities;
using Uni.WebApi.Models.ModelDto;
using Uni.WebApi.Models.Models;
using Uni.WebApi.Models.Requests;

namespace Uni.WebApi.ApplicationCore.Services
{
    public class FacultyService : IFacultyService
    {
        private readonly IUnitOfWork _unitOfWork;
        private readonly IMapper _mapper;
        private readonly UrlsOptions _urlOptions;

        public FacultyService(
            IUnitOfWork unitOfWork,
            IOptions<UrlsOptions> urlOptions,
            IMapper mapper)
        {
            this._mapper = mapper;
            this._unitOfWork = unitOfWork;
            this._urlOptions = urlOptions.Value;
        }

        public async Task<ServiceResult<FacultyDto>> CreateFacultyAsync(AddFacultyRequest request)
           {
               if (!await this._unitOfWork.UniversityRepository.IsExistById(request.UniversityId))
               {
                   return ServiceResult<FacultyDto>.Fail(EOperationResult.EntityNotFound,
                       "University with this Id doesn't exist");
               }

               var newFaculty = new Faculty()
               {
                   FullTitle = request.FullTitle,
                   ShortTitle = request.ShortTitle,
                   Description = request.Description,
                   UniversityId = request.UniversityId,
                   Avatar = request.Avatar
               };

               if (string.IsNullOrEmpty(newFaculty.Avatar))
               {
                   newFaculty.Avatar = this._urlOptions.ServerUrl + DefaultImagesConstants.DefaultImage;
               }

               this._unitOfWork.FacultyRepository.Add(newFaculty);

               await this._unitOfWork.CommitAsync();

               return ServiceResult<FacultyDto>.Ok(this._mapper.Map<Faculty, FacultyDto>(newFaculty));
           }

        public async Task<ServiceResult<IEnumerable<FacultyDto>>> GetListOfFacultiesAsync(int universityId, int skip, int take)
        {
            IEnumerable<FacultyDto> result = this._mapper.Map<IEnumerable<Faculty>, IEnumerable<FacultyDto>>(
                await this._unitOfWork.FacultyRepository.GetFacultiesByUniversityAsync(universityId, skip, take));

            return ServiceResult<IEnumerable<FacultyDto>>.Ok(result);
        }
    }
}