using System.Net;
using System.Threading.Tasks;
using FluentEmail.Core;
using FluentEmail.Core.Models;
using FluentEmail.Razor;
using FluentEmail.SendGrid;
using Microsoft.Extensions.Options;
using SendGrid;
using SendGrid.Helpers.Mail;
using Uni.WebApi.ApplicationCore.Helpers.Contract;
using Uni.WebApi.ApplicationCore.Services.Shared.Contract;
using Uni.WebApi.Common.Options;
using Uni.WebApi.Models.Models;

namespace Uni.WebApi.ApplicationCore.Services.Shared
{
    public class EmailService : IEmailService
    {
        private readonly SendGridOptions _sendGridOptions;
        private readonly IEmailTemplatePicker _emailTemplatePicker;

        public EmailService(IOptions<SendGridOptions> sendGridOptions,
            IEmailTemplatePicker emailTemplatePicker)
        {
            this._emailTemplatePicker = emailTemplatePicker;
            this._sendGridOptions = sendGridOptions.Value;
            Email.DefaultSender = new SendGridSender(this._sendGridOptions.ApiKey);
            Email.DefaultRenderer = new RazorRenderer();
        }

        public async Task<ServiceResult<SendResponse>> SendUserValidationEmailAsync(string email, string username, string link)
        {
            var client = new SendGridClient(this._sendGridOptions.ApiKey);
            var from = new EmailAddress(this._sendGridOptions.EmailFrom, "Admin");
            var subject = "Email confirmation";
            var to = new EmailAddress(email, username);
            var plainTextContent = "";
            
            string tempalate = this._emailTemplatePicker.GetRegistrationHtml( username, link);
            
            var htmlContent = tempalate;
            var msg = MailHelper.CreateSingleEmail(from, to, subject, plainTextContent, htmlContent);
            var response = await client.SendEmailAsync(msg);

            if (response.StatusCode == HttpStatusCode.Accepted)
            {
                return ServiceResult<SendResponse>.Ok();
            }
            else
            {
                return ServiceResult<SendResponse>.Fail(EOperationResult.SendEmailError, string.Join(", ", response.Body));
            }
        }
    }
}