using System.Collections.Generic;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Uni.WebApi.ApplicationCore.Services.Contract;
using Uni.WebApi.Models.Enums;
using Uni.WebApi.Models.ModelDto;
using Uni.WebApi.Models.Requests;
using Uni.WebApi.Web.Helpers.Mappers;

namespace Uni.WebApi.Controllers
{
    [ApiVersion("1.0")]
    [Route("/v{api-version:apiVersion}/[controller]")]
    [ApiController]
    public class CitiesController : BaseController
    {
        private readonly IServiceResultMapper _viewMapper;
        private readonly ICityService _cityService;

        public CitiesController(
            IServiceResultMapper viewMapper,
            ICityService cityService)
        {
            this._viewMapper = viewMapper;
            this._cityService = cityService;
        }

        [HttpPost]
        [Authorize(Roles = nameof(ERoleType.Admin))]
        public async Task<ActionResult<CityDto>> AddCityAsync([FromBody] AddCityRequest request)
            => this._viewMapper.ServiceResultToContentResult(
                await this._cityService.CreateCityAsync(request));

        [HttpGet]
        public async Task<ActionResult<IEnumerable<CityDto>>> GetCititesAsync(int countryId)
        => this._viewMapper.ServiceResultToContentResult(
                await this._cityService.GetListOfCitiesAsync(countryId));
    }
}
