using System.Collections.Generic;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Uni.WebApi.ApplicationCore.Services.Contract;
using Uni.WebApi.Models.Enums;
using Uni.WebApi.Models.ModelDto;
using Uni.WebApi.Models.Requests;
using Uni.WebApi.Web.Helpers.Mappers;

namespace Uni.WebApi.Controllers
{
    [ApiVersion("1.0")]
    [Route("/v{api-version:apiVersion}/[controller]")]
    [ApiController]
    public class FacultiesController : BaseController
    {
        private readonly IServiceResultMapper _viewMapper;
        private readonly IFacultyService _facultyService;

        public FacultiesController(
            IServiceResultMapper viewMapper,
            IFacultyService facultyService)
        {
            this._viewMapper = viewMapper;
            this._facultyService = facultyService;
        }

        [HttpPost]
        [Authorize(Roles = nameof(ERoleType.Admin))]
        public async Task<ActionResult<FacultyDto>> AddFacultyAsync([FromBody] AddFacultyRequest request)
            => this._viewMapper.ServiceResultToContentResult(
                await this._facultyService.CreateFacultyAsync(request));

        [HttpGet]
        public async Task<ActionResult<IEnumerable<FacultyDto>>> GetFacultiesAsync(int universityId, int skip = 0, int take = 10)
        => this._viewMapper.ServiceResultToContentResult(
                await this._facultyService.GetListOfFacultiesAsync(universityId,  skip, take));
    }
}
