using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Uni.WebApi.ApplicationCore.Services.Contract;
using Uni.WebApi.Models.Enums;
using Uni.WebApi.Web.Helpers.Mappers;

namespace Uni.WebApi.Controllers
{
    [ApiVersion("1.0")]
    [Route("/v{api-version:apiVersion}/[controller]")]
    [ApiController]
    public class FilesController : BaseController
    {
        private readonly IServiceResultMapper _viewMapper;
        private readonly IFileService _fileService;
        
        public FilesController(
            IServiceResultMapper viewMapper,
            IFileService fileService)
        {
            this._viewMapper = viewMapper;
            this._fileService = fileService;
        }
        
        [HttpPost("upload-image")]
        [DisableRequestSizeLimit]
        [Authorize(Roles = nameof(ERoleType.Admin) + "," + nameof(ERoleType.Student))]
        public async Task<IActionResult> UploadImageAsync(IFormFile file)
            => this._viewMapper.ServiceResultToContentResult
                (await this._fileService.UploadImageAsync(file));

        [HttpPost("upload-file")]
        [DisableRequestSizeLimit]
        [Authorize(Roles = nameof(ERoleType.Admin) + "," + nameof(ERoleType.Student))]
        public async Task<IActionResult> UploadFileAsync(IFormFile file)
            => this._viewMapper.ServiceResultToContentResult
                (await this._fileService.UploadFileAsync(file, UserId));
    }
}