using System.Collections.Generic;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Uni.WebApi.ApplicationCore.Services.Contract;
using Uni.WebApi.Models.Enums;
using Uni.WebApi.Models.ModelDto;
using Uni.WebApi.Models.Requests;
using Uni.WebApi.Web.Helpers.Mappers;

namespace Uni.WebApi.Controllers
{
    [ApiVersion("1.0")]
    [Route("/v{api-version:apiVersion}/[controller]")]
    [ApiController]
    public class GroupsController : BaseController
    {
        private readonly IServiceResultMapper _viewMapper;
        private readonly IGroupService _groupService;

        public GroupsController(
            IServiceResultMapper viewMapper,
            IGroupService groupService)
        {
            this._viewMapper = viewMapper;
            this._groupService = groupService;
        }

        [HttpPost]
        [Authorize(Roles = nameof(ERoleType.Admin))]
        public async Task<ActionResult<GroupDto>> AddGroupAsync([FromBody] AddGroupRequest request)
            => this._viewMapper.ServiceResultToContentResult(
                await this._groupService.CreateGroupAsync(request));

        [HttpGet]
        public async Task<ActionResult<IEnumerable<GroupDto>>> GetFacultiesAsync(int facultyId)
        => this._viewMapper.ServiceResultToContentResult(
                await this._groupService.GetListOfGroupsAsync(facultyId));
    }
}
