using System.Threading.Tasks;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Uni.WebApi.ApplicationCore.Services.Contract;
using Uni.WebApi.Models.Enums;
using Uni.WebApi.Models.ModelDto;
using Uni.WebApi.Models.Requests;
using Uni.WebApi.Web.Helpers.Mappers;

namespace Uni.WebApi.Controllers
{    
    [ApiVersion("1.0")]
    [Route("/v{api-version:apiVersion}/[controller]")]
    [ApiController]
    public class TeachersController
    {
        private readonly IServiceResultMapper _viewMapper;
        private readonly ITeacherService _teacherService;
        
        public TeachersController(ITeacherService teacherService, IServiceResultMapper viewMapper)
        {
            this._teacherService = teacherService;
            this._viewMapper = viewMapper;
        }
        
        [HttpPost]
        [Authorize(Roles = nameof(ERoleType.Admin))]
        public async Task<ActionResult<TeacherDto>> AddTeacherAsync([FromBody] AddTeacherRequest request)
            => this._viewMapper.ServiceResultToContentResult(
                await this._teacherService.CreateTeacherAsync(request));
    }
}